// 支持trim的html dom类型
const domTypes = ['input', 'textarea']
// trimHandle处理后，再一次通知的监听事件类型
const informEventTypes = ['input', 'change']
// 键盘输入时触发时机的事件类型
const diyEventType = 'free_trim_change'
// dom监听事件类型
const updateEventTypes = ['change', diyEventType]
// 自定义能发事件
const dispatchEvent = (el, events) => {
    events = Array.isArray(events) ? events : [events]
    events.forEach(event => el.dispatchEvent(new CustomEvent(event)))
}
// trim处理
const trimHandle = el => {
    return event => {
        const oldValue = event.target.value
        const newVal = oldValue.trim()
        if (newVal !== oldValue) {
            event.target.value = newVal
            //通知在当前dom上的其它监听事件
            dispatchEvent(el, informEventTypes)
        }
    }
}
// 循环处理dom事件
function loopHandle(el, func) {
    domTypes.forEach(domType => {
        if (!el || !el.querySelectorAll) return
        const doms = el.querySelectorAll(domType)
        Array.from(doms).forEach(dom => func(dom))
    })
}

// 添加监听事件
function addEventListener(el) {
    loopHandle(el, dom => {
        if (dom.hasAddTrimEvent) return
        dom.hasAddTrimEvent = true
        dom._trimHandle = trimHandle(dom)
        updateEventTypes.forEach(event => dom.addEventListener(event, dom._trimHandle))
        // 没有触发过的dom，自行做发一次自定义事件，处理初始化值
        setTimeout(()=>{
            dispatchEvent(dom, diyEventType)
        },0)
    })
}
// 移除监听事件
function removeEventListener(el) {
    loopHandle(el, dom => {
        if (!dom.hasAddTrimEvent) return
        dom.hasAddTrimEvent = false
        updateEventTypes.forEach(event => dom.removeEventListener(event, dom._trimHandle))
    })
}
export default {
    install: Vue => {
        Vue.mixin({
            mounted() {
                addEventListener(this.$el)
            },
            beforeDestroy() {
                removeEventListener(this.$el)
            },
        })
    },
}